#include "stdafx.h"

const int ID_EDTVALUE	= 5000;
const int ID_BTNOK		= 5001;
const int ID_BTNCANCEL	= 5002;
const int ID_BTNRESET	= 5003;
const int ID_BTNADDMARK	= 5004;
const int ID_BTNDELMARK	= 5005;

extern bool g_bDrag;
extern int g_iXDrag;
extern int g_iYDrag;

//-----------------------------------------------------------------------------
class CVwHistogram : public CDialog { 
	public : 
		//-- Window handle 
		HWND m_hWnd;
		bool m_bSetup;

		//-- Controls for Top of window
		CDlgEdit * m_edtValue;
		CDlgControl * m_btnDeleteMarker;
		CDlgControl * m_btnAddMarker;
		CDlgControl * m_btnOK;
		CDlgControl * m_btnCancel;
		CDlgControl * m_btnReset;
		
		//-- data 	
		RECT m_rctGraph;					// Histogram plotting area 
		CHistogram * m_pHistogram;			// Scene container
		int GraphWidth() {	return m_rctGraph.right-m_rctGraph.left; } 
		int GraphHeight() { return m_rctGraph.bottom-m_rctGraph.top; } 
		
		//-----------------------------------------------------------------------
		CVwHistogram() { 
			m_pHistogram = NULL;
			m_bSetup=false;
			}

		~CVwHistogram() { 
			m_bSetup=false;
			if( m_pHistogram != NULL ) 
				delete m_pHistogram;
			}
		
		//-----------------------------------------------------------------------
		void InitializeControls( ) {
			const int IC_HGT=20;
			const int IC_SEP=22;
			const int IC_CHWID=8;
			const int IC_TOP=10;

			//-- Text Labels
			int iLeft=100;
			CreateLabel( iLeft, IC_TOP+IC_SEP*0, 100, IC_HGT, IDC_STATIC, NULL, 0, SS_RIGHT )->SetText( "Value:" );
			iLeft=200;
			m_edtValue = CreateEdit( iLeft, IC_TOP+IC_SEP*0, 100 , IC_HGT, ID_EDTVALUE , "Position of current marker", 0 );
			m_edtValue->SetTextLimit( 16 );	

			iLeft=350;
			(m_btnOK			= CreateButton( iLeft, IC_TOP+IC_SEP*0, 99, IC_HGT, ID_BTNOK		, NULL, "Accept level settings and close"))->SetText("OK");
			(m_btnCancel		= CreateButton( iLeft, IC_TOP+IC_SEP*1, 99, IC_HGT, ID_BTNCANCEL	, NULL, "Cancel setting levels"))->SetText("Cancel");
			(m_btnReset			= CreateButton( iLeft, IC_TOP+IC_SEP*2, 99, IC_HGT, ID_BTNRESET	, NULL, "Reset to initial levels"))->SetText("Reset");
			(m_btnAddMarker		= CreateButton( iLeft, IC_TOP+IC_SEP*3, 99, IC_HGT, ID_BTNADDMARK, NULL, "Add a new marker on next click"))->SetText("Add Marker");
			(m_btnDeleteMarker	= CreateButton( iLeft, IC_TOP+IC_SEP*4, 99, IC_HGT, ID_BTNDELMARK, NULL, "Delete Selected Marker"))->SetText("Delete Mark");
			}
	
		//-----------------------------------------------------------------------
		void CauseRedraw( ) { 			
			InvalidateRect( m_hWnd, &m_rctGraph, true );
			}

		//-----------------------------------------------------------------------
		void Draw( HDC hdc ) {
			m_pHistogram->Draw( hdc, m_rctGraph );
			}		
			
		//-----------------------------------------------------------------------
		BOOL OnClickGraph( int iX, int iY ) {	
			//TODO - return TRUE if clicked on draggable object 
			return false;
			}
			
		BOOL OnDrag( int iX, int iY ) {	
			//TODO 
			return false;
			}

		BOOL OnDragRelease( int iX, int iY ) {	
			//TODO 
			return false;
			}

		//-----------------------------------------------------------------------
		BOOL OnKey( DWORD dwKey ) { 
			switch( dwKey ) { 
				case 37 :				// left
					//TODO
					break;
				case 39 :				// right
					//TODO 
					break;
				case 38 :				// Up
					//TODO 
					break;
				case 40 :				// down
					//TODO 
					break;
				case 36 :				// Home (CW)
					//TODO 
					break;
				case 35 :				// End (CCW)
					//TODO 
					break;
					
				default : 
					return FALSE;
				}
						
			CauseRedraw();	
			return TRUE;
			} 		
			
		//-----------------------------------------------------------------------
		void OnSetSize( int iWid, int iHgt ) { 
			iWid=max(IDLHISTBINCOUNT+100,iWid);
			iHgt=max(400,iHgt);
			SetRect( &m_rctGraph, iWid/2-(IDLHISTBINCOUNT+100)/2, 
					200, iWid/2+(IDLHISTBINCOUNT+100)/2, iHgt-75 );
			if( m_bSetup )
				CauseRedraw();
			}
			
		//-----------------------------------------------------------------------
		void OnValueChanged() { 
			//TODO 
			}
			
		//-----------------------------------------------------------------------
		void OnBtnOK() {
			//TODO 
			}
		void OnBtnCancel() {
			//TODO 
			}
		void OnBtnReset() {
			//TODO 
			}
		void OnBtnAddMarker() {
			//TODO 
			}
		void OnBtnDelMarker() {
			//TODO 
			}
	};

//-----	
LRESULT CALLBACK HistogramWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {
	PAINTSTRUCT ps;
	CVwHistogram * pVw;
	switch( message ) {
		case WM_CREATE : 
			pVw = new CVwHistogram( );
			pVw->m_hDlg=hWnd; 
			pVw->m_hWnd=hWnd;
			SetWindowLong( hWnd, 0,  (long) pVw ) ;
			pVw->InitializeControls();
			return 0;		

		case WM_COMMAND : 
			pVw = (CVwHistogram *)GetWindowLong( hWnd, 0 );
			switch( wParam ) { 
				//-- value changed
				case (EN_CHANGE<<16)|ID_EDTVALUE : 
					pVw->OnValueChanged();
					break;
				
				//-- Button clicked 
				case ID_BTNOK	:	
					pVw->OnBtnOK();
					break;
				case ID_BTNCANCEL	:	
					pVw->OnBtnCancel();
					break;
				case ID_BTNRESET	:	
					pVw->OnBtnReset();
					break;
				case ID_BTNADDMARK	:	
					pVw->OnBtnAddMarker();
					break;
				case ID_BTNDELMARK	:	
					pVw->OnBtnDelMarker();
					break;
				}	
			return 0;

		case WM_IDLHIST_OPEN : 
			pVw = (CVwHistogram *)GetWindowLong( hWnd, 0 );
			pVw->m_pHistogram = (CHistogram *)lParam;
			pVw->m_bSetup=true;
			break;

		case WM_MDIACTIVATE :		// Change the menu when this window is activated.
			return 0;		
			
		case WM_KEYDOWN : 
			pVw = (CVwHistogram *)GetWindowLong( hWnd, 0 );
			pVw->OnKey( wParam );
			return 0;

		case WM_PAINT : {
			HDC hdc = BeginPaint(hWnd, &ps);

			//----- Manual Drawing 
			pVw = (CVwHistogram *) GetWindowLong(  hWnd, 0 );
			pVw->Draw( hdc );
			
			//----- Done
			EndPaint(hWnd, &ps);
			}
			return 0;

		case WM_SIZE : 
			pVw = (CVwHistogram *)GetWindowLong( hWnd, 0 );
			pVw->OnSetSize( LOWORD(lParam), HIWORD(lParam) );
			break;

		case WM_DESTROY : 
			pVw = (CVwHistogram *)GetWindowLong( hWnd, 0 );
			delete pVw;
			SetWindowLong( hWnd, 0,  0 ) ;
			return 0;	

		case WM_LBUTTONDOWN : {
			//TODO 
			pVw = (CVwHistogram *)GetWindowLong( hWnd, 0 );
			int iX = LOWORD(lParam);
			int iY = HIWORD(lParam);
			if( iY >= pVw->m_rctGraph.top 
				&& iY <= pVw->m_rctGraph.bottom 
				&& iX >= pVw->m_rctGraph.left 
				&& iX <= pVw->m_rctGraph.right 
				&& pVw->OnClickGraph( iX, iY ) ) { 
				
				SetFocus( pVw->m_hWnd );
				SetCapture( pVw->m_hWnd );
				g_bDrag=true;
				g_iXDrag=iX;
				g_iYDrag=iY;
				}
			}
			return 0;
			
		case WM_MOUSEMOVE :			//-- Mouse move during drag operation
			if( g_bDrag ) { 
				pVw = (CVwHistogram *)GetWindowLong( hWnd, 0 );
				int iX=LOWORD(lParam);
				int iY=HIWORD(lParam);
				pVw->OnDrag( iX, iY );
				g_iXDrag=iX;
				g_iYDrag=iY;
				}
			return 0;
			
		case WM_LBUTTONUP :			//-- end of drag operation
			if( g_bDrag ) { 
				pVw = (CVwHistogram *)GetWindowLong( hWnd, 0 );
				SetCapture( NULL );
				g_bDrag=false;
				pVw->OnDragRelease( LOWORD(lParam), HIWORD(lParam) );
				}
			return 0;
			
		case WM_GETMINMAXINFO : 
			((MINMAXINFO*)lParam)->ptMinTrackSize.x=IDLHISTBINCOUNT+100;
			((MINMAXINFO*)lParam)->ptMinTrackSize.y=400;
			return 0;

		}
		
	return DefMDIChildProc( hWnd, message, wParam, lParam);
	}

